@@ -1,8 +1,72 @@
+## Copyright 2013 Stefan Widgren and Maria Noremark,
+## National Veterinary Institute, Sweden
+##
+## Licensed under the EUPL, Version 1.1 or - as soon they
+## will be approved by the European Commission - subsequent
+## versions of the EUPL (the "Licence");
+## You may not use this work except in compliance with the
+## Licence.
+## You may obtain a copy of the Licence at:
+##
+## http://ec.europa.eu/idabc/eupl
+##
+## Unless required by applicable law or agreed to in
+## writing, software distributed under the Licence is
+## distributed on an "AS IS" basis,
+## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
+## express or implied.
+## See the Licence for the specific language governing
+## permissions and limitations under the Licence.
+
 ##' \code{InDegree}
 ##'
 ##' The number of herds with direct movements of animals to the root herd
 ##' during the defined time window used for tracing.
 ##'
+##'
+##' The time period used for \code{InDegree} can either be specified
+##' using \code{tEnd} and \code{days} or \code{inBegin} and \code{inEnd}.
+##'
+##' If using \code{tEnd} and \code{days}, the time period for ingoing
+##' contacts ends at \code{tEnd} and starts at \code{days} prior to
+##' \code{tEnd}. The indegree will be calculated for each combination
+##' of \code{root}, \code{tEnd} and \code{days}.
+##'
+##' An alternative way is to use \code{inBegin} and \code{inEnd}.  The
+##' time period for ingoing contacts starts at inBegin and ends at
+##' inEndDate. The vectors \code{root} \code{inBegin}, \code{inEnd}
+##' must have the same lengths and the indegree will be calculated for
+##' each index of them.
+##'
+##' The movements in \code{InDegree} is a \code{data.frame}
+##' with the following columns:
+##' \describe{
+##'
+##'   \item{source}{
+##'     an integer or character identifier of the source holding.
+##'   }
+##'
+##'   \item{destination}{
+##'     an integer or character identifier of the destination holding.
+##'   }
+##'
+##'   \item{t}{
+##'     the Date of the transfer
+##'   }
+##'
+##'   \item{id}{
+##'     an optional character vector with the identity of the animal.
+##'   }
+##'
+##'   \item{n}{
+##'     an optional numeric vector with the number of animals moved.
+##'   }
+##'
+##'   \item{category}{
+##'     an optional character or factor with category of the animal e.g. Cattle.
+##'   }
+##' }
+##'
 ##' @name InDegree-methods
 ##' @aliases InDegree
 ##' @aliases InDegree-methods
@@ -23,17 +87,22 @@
 ##'   }
 ##'
 ##'   \item{\code{signature(x = "data.frame")}}{
-##'     Get the InDegree for a data.frame with movements, see examples.
+##'     Get the InDegree for a data.frame with movements, see details and examples.
 ##'   }
 ##' }
 ##' @seealso \code{\link{NetworkSummary}}
 ##' @param x a ContactTrace object, or a list of ContactTrace objects
 ##' or a \code{data.frame} with movements of animals between holdings,
-##' see \code{\link{TraceDateInterval}} for details.
-##' @param root vector of roots to perform contact tracing on.
-##' @param tEnd the last date to include ingoing movements
+##' see \code{\link{Trace}} for details.
+##' @param root vector of roots to calculate indegree for.
+##' @param tEnd the last date to include ingoing movements. Defaults
+##' to \code{NULL}
 ##' @param days the number of previous days before tEnd to include
-##' ingoing movements
+##' ingoing movements. Defaults to \code{NULL}
+##' @param inBegin the first date to include ingoing
+##' movements. Defaults to \code{NULL}
+##' @param inEnd the last date to include ingoing movements. Defaults
+##' to \code{NULL}
 ##' @return A \code{data.frame} with the following columns:
 ##' \describe{
 ##'   \item{root}{
@@ -75,38 +144,42 @@
 ##' ## Load data
 ##' data(transfers)
 ##'
-##' ## Perform contact tracing
+##' ## Perform contact tracing using tEnd and days
 ##' contactTrace <- Trace(movements=transfers,
 ##'                       root=2645,
 ##'                       tEnd='2005-10-31',
-##'                       days=90)
+##'                       days=91)
 ##'
-##' InDegree(contactTrace)
+##' ## Calculate indegree from a ContactTrace object
+##' id.1 <- InDegree(contactTrace)
+##'
+##' ## Calculate indegree using tEnd and days
+##' id.2 <- InDegree(transfers,
+##'                  root=2645,
+##'                  tEnd='2005-10-31',
+##'                  days=91)
+##'
+##' ## Check that the result is identical
+##' identical(id.1, id.2)
 ##'
 ##' \dontrun{
-##' ## Perform contact tracing for all included herds
+##' ## Calculate indegree for all included herds
 ##' ## First extract all source and destination from the dataset
 ##' root <- sort(unique(c(transfers$source,
 ##'                       transfers$destination)))
 ##'
-##' ## Perform contact tracing
+##' ## Calculate indegree
 ##' result <- InDegree(transfers,
 ##'                    root=root,
 ##'                    tEnd='2005-10-31',
-##'                    days=90)
+##'                    days=91)
 ##' }
 ##'
 setGeneric('InDegree',
            signature = 'x',
            function(x, ...) standardGeneric('InDegree'))
 
-## For internal use
-setGeneric('in_degree',
-           signature = 'x',
-           function(x) standardGeneric('in_degree'))
-
-## For internal use
-setMethod('in_degree',
+setMethod('InDegree',
           signature(x = 'Contacts'),
           function(x)
       {
@@ -146,20 +219,34 @@ setMethod('InDegree',
           signature(x = 'data.frame'),
           function(x,
                    root,
-                   tEnd,
-                   days)
+                   tEnd = NULL,
+                   days = NULL,
+                   inBegin = NULL,
+                   inEnd = NULL)
       {
-          if(any(missing(x),
-                 missing(root),
-                 missing(tEnd),
-                 missing(days))) {
+          if(missing(root)) {
               stop('Missing parameters in call to InDegree')
           }
 
-          return(NetworkSummary(x, root, tEnd, days)[, c('root',
-                                                         'inBegin',
-                                                         'inEnd',
-                                                         'inDays',
-                                                         'inDegree')])
+          if(all(is.null(tEnd), is.null(days))) {
+              outBegin <- inBegin
+              outEnd <- outBegin
+          } else {
+              outBegin <- NULL
+              outEnd <- NULL
+          }
+
+          return(NetworkSummary(x,
+                                root,
+                                tEnd,
+                                days,
+                                inBegin,
+                                inEnd,
+                                outBegin,
+                                outEnd)[, c('root',
+                                            'inBegin',
+                                            'inEnd',
+                                            'inDays',
+                                            'inDegree')])
       }
 )
